@@ -87,6 +87,9 @@ namespace FSpot {
 		
 		public void RangeChanged (object sender, EventArgs args)
 		{
+			if (!view.CurrentPhotoValid ())
+				return;
+
 			if (thread != null && thread.IsAlive)
 				thread.Abort ();
 			
@@ -96,8 +99,12 @@ namespace FSpot {
 		
 		public void Save ()
 		{
+			if (!view.CurrentPhotoValid ())
+				return;
+
 			Console.WriteLine ("Saving....");
 			Photo photo = view.Query.Photos[view.CurrentPhoto];
+			Exif.ExifData data = new Exif.ExifData (photo.DefaultVersionPath);
 			
 			uint version = photo.DefaultVersionId;
 			if (version == Photo.OriginalVersionId) {
@@ -117,14 +124,14 @@ namespace FSpot {
 			
 			try {
 				string version_path = photo.GetVersionPath (version);
-				
-				final.Savev (version_path, "jpeg", null, null);
-				PhotoStore.GenerateThumbnail (version_path);
+
+				PixbufUtils.SaveJpeg (final, version_path, 95, data);
+				ThumbnailGenerator.Create (version_path).Dispose ();
 				photo.DefaultVersionId = version;
 				view.Query.Commit (view.CurrentPhoto);
 			} catch (GLib.GException ex) {
 				// FIXME error dialog.
-				Console.WriteLine ("error {0}", ex);
+				Console.WriteLine (ex.ToString ());
 			}
 			
 			Console.WriteLine ("Saving....");
@@ -150,9 +157,11 @@ namespace FSpot {
 
 		private void HandlePhotoChanged (PhotoImageView view)
 		{
-			AdjustedPixbuf = PhotoLoader.LoadAtMaxSize (view.Query.Photos [view.CurrentPhoto], 300, 300);
-			ScaledPixbuf = AdjustedPixbuf.Copy ();			
-			RangeChanged (null, null);
+			if (view.CurrentPhotoValid ()) {
+				AdjustedPixbuf = PhotoLoader.LoadAtMaxSize (view.Query.Photos [view.CurrentPhoto], 300, 300);
+				ScaledPixbuf = AdjustedPixbuf.Copy ();			
+				RangeChanged (null, null);
+			}
 		}
 		
 		private void HandleCancelClicked (object sender, EventArgs args)
@@ -189,12 +198,14 @@ namespace FSpot {
 		private void AttachInterface (string ui_path)
 		{
 			Glade.XML xml = new Glade.XML (null, "f-spot.glade", ui_path, null);
-
+			/*
+			if (!view.CurrentPhotoValid ())
+				return;
+			*/
+			HandlePhotoChanged (view);
 			view.PhotoChanged += HandlePhotoChanged;
 			xml.Autoconnect (this);
-			
-			AdjustedPixbuf = PhotoLoader.LoadAtMaxSize (view.Query.Photos [view.CurrentPhoto], 300, 300);
-			ScaledPixbuf = AdjustedPixbuf.Copy ();
+
 
 			hist = new FSpot.Histogram ();
 			expose_timeout = new FSpot.Delay (new GLib.IdleHandler (this.QueueDraw));
@@ -208,7 +219,6 @@ namespace FSpot {
 			hist.Color [3] = 0xff;
 			#endif
 
-			hist.FillValues (AdjustedPixbuf);
 			histogram_image.Pixbuf = hist.GeneratePixbuf ();
 
 			brightness_spinbutton.Adjustment = brightness_scale.Adjustment;
