@@ -3,11 +3,12 @@ using Gtk;
 using Gdk;
 using GLib;
 
+
 namespace FSpot {
 	public class GroupSelector : Fixed {
 		internal static GType groupSelectorGType;
 
-		int border = 12;
+		int border = 6;
 		int box_spacing = 2;
 		int box_top_padding = 6;
 		public static int MIN_BOX_WIDTH = 20;
@@ -16,15 +17,16 @@ namespace FSpot {
 		private Limit min_limit;
 		private Limit max_limit;
 
-#if USE_BUTTONS
 		private Gtk.Button left;
 		private Gtk.Button right;
-#endif
+		private Delay left_delay;
+		private Delay right_delay;
 
 		private Gdk.Window event_window;
 
 		public Gdk.Rectangle background;
 		public Gdk.Rectangle legend;
+		public Gdk.Rectangle action;
 
 		int    box_count_max;
 		int [] box_counts = new int [0];
@@ -39,9 +41,10 @@ namespace FSpot {
 
 				adaptor = value;
 				has_limits = adaptor is FSpot.ILimitable;				
+
 				if (has_limits) {
-				    min_limit.SetPosition (0);
-				    max_limit.SetPosition (adaptor.Count () - 1);
+				    min_limit.SetPosition (0, false);
+				    max_limit.SetPosition (adaptor.Count () - 1, false);
 				}
 
 				adaptor.Changed += HandleAdaptorChanged;
@@ -54,40 +57,37 @@ namespace FSpot {
 
 		private void HandleAdaptorChanged (GroupAdaptor adaptor)
 		{
-				int [] box_values = new int [adaptor.Count ()];
-
-				if (tick_layouts != null) {
-					foreach (Pango.Layout l in tick_layouts) {
-						if (l != null)
-							l.Dispose ();
-					}
+			bool size_changed = box_counts.Length != adaptor.Count ();
+			int [] box_values = new int [adaptor.Count ()];
+			
+			if (tick_layouts != null) {
+				foreach (Pango.Layout l in tick_layouts) {
+					if (l != null)
+						l.Dispose ();
 				}
-				tick_layouts = new Pango.Layout [adaptor.Count ()];
-
-				int i = 0;
-				while (i < adaptor.Count ()) {
-					box_values [i] = adaptor.Value (i);
-					string label = adaptor.TickLabel (i);
-					if (label != null) {
-						tick_layouts [i] = CreatePangoLayout (label);
-					}
-					i++;
+			}
+			tick_layouts = new Pango.Layout [adaptor.Count ()];
+			
+			int i = 0;
+			while (i < adaptor.Count ()) {
+				box_values [i] = adaptor.Value (i);
+				string label = adaptor.TickLabel (i);
+				if (label != null) {
+					tick_layouts [i] = CreatePangoLayout (label);
 				}
-
-				if (glass.Position >= adaptor.Count())
-					glass.SetPosition (adaptor.Count() - 1);
-
-				Counts = box_values;
-
-				if (has_limits) {
-					if (min_limit.Position > adaptor.Count ())
-						min_limit.SetPosition (0);
-				        
-				     
-					if (max_limit.Position > adaptor.Count ())
-						max_limit.SetPosition (adaptor.Count () - 1);
-				}
-				this.QueueDraw ();
+				i++;
+			}
+			
+			if (glass.Position >= adaptor.Count())
+				glass.SetPosition (adaptor.Count() - 1);
+			
+			Counts = box_values;
+			
+			if (has_limits && size_changed) {
+				min_limit.SetPosition (0, false);
+				max_limit.SetPosition (adaptor.Count () - 1, false);
+			}
+			this.QueueDraw ();
 		}
 		
 		private int [] Counts {
@@ -128,12 +128,34 @@ namespace FSpot {
 			}
 			set {
 				scroll_offset = value;
+				
+				int total_width = (int)(box_counts.Length * BoxWidth);
+
+				if (total_width + scroll_offset < background.Width)
+					scroll_offset = background.Width - total_width;
+				
+				if (total_width <= background.Width)
+					scroll_offset = 0;
+
+				UpdateButtons ();
+
 				if (Visible)
 					GdkWindow.InvalidateRect (Allocation, false);
 			}
 		}
 		
-		static bool BoxTest (Rectangle bounds, double x, double y) 
+		private void UpdateButtons () {
+			left.Sensitive = (scroll_offset < 0);
+			right.Sensitive = (box_counts.Length * BoxWidth > background.Width - scroll_offset);
+
+			if (!left.Sensitive && left_delay.IsPending) 
+				left_delay.Stop ();
+
+			if (!right.Sensitive && right_delay.IsPending)
+				right_delay.Stop ();
+		}
+
+		static bool IsInside (Rectangle bounds, double x, double y) 
 		{
 			if (x >= bounds.X && 
 			    x < bounds.X + bounds.Width && 
@@ -193,7 +215,7 @@ namespace FSpot {
 		private bool BoxHit (double x, double y, out int position) 
 		{
 			if (BoxXHit (x, out position)) {
-				if (BoxTest (BoxBounds (position), x, y))
+				if (IsInside (BoxBounds (position), x, y))
 					return true;
 
 				position++;
@@ -203,8 +225,8 @@ namespace FSpot {
 
 		protected override bool OnButtonPressEvent (Gdk.EventButton args)
 		{
-			double x = args.X + Allocation.X;
-			double y = args.Y + Allocation.Y;
+			double x = args.X + action.X;
+			double y = args.Y + action.Y;
 
 			if (glass.IsInside (x, y)) {
 				glass.StartDrag (x, y, args.Time);
@@ -226,8 +248,8 @@ namespace FSpot {
 		
 		protected override bool OnButtonReleaseEvent (Gdk.EventButton args) 
 		{
-			double x = args.X + Allocation.X;
-			double y = args.Y + Allocation.Y;
+			double x = args.X + action.X;
+			double y = args.Y + action.Y;
 
 			if (glass.Dragging) {
 				glass.EndDrag (x, y);
@@ -247,8 +269,8 @@ namespace FSpot {
 
 		protected override bool OnMotionNotifyEvent (Gdk.EventMotion args) 
 		{
-			double x = args.X + Allocation.X;
-			double y = args.Y + Allocation.Y;
+			double x = args.X + action.X;
+			double y = args.Y + action.Y;
 
 			//Rectangle box = glass.Bounds ();
 			//Console.WriteLine ("please {0} and {1} in box {2}", x, y, box);
@@ -277,11 +299,13 @@ namespace FSpot {
 			
 			WindowAttr attr = WindowAttr.Zero;
 			attr.WindowType = Gdk.WindowType.Child;
+			
+			
 
-			attr.X = Allocation.X;
-			attr.Y = Allocation.Y;
-			attr.Width = Allocation.Width;
-			attr.Height = Allocation.Height;
+			attr.X = action.X;
+			attr.Y = action.Y;
+			attr.Width = action.Width;
+			attr.Height = action.Height;
 			attr.Wclass = WindowClass.InputOnly;
 			attr.EventMask = (int) Events;
 			attr.EventMask |= (int) (EventMask.ButtonPressMask | 
@@ -307,6 +331,7 @@ namespace FSpot {
 				case RangeType.Fixed:
 					return background.Width / (double) 12;
 				case RangeType.Min:
+					return Math.Max (MIN_BOX_WIDTH, background.Width / (double) box_counts.Length);
 				default:
 					return (double) MIN_BOX_WIDTH;
 				}
@@ -476,6 +501,11 @@ namespace FSpot {
 			}
 
 			public void SetPosition (int position)
+			{
+				SetPosition (position, true);
+			}
+
+			public void SetPosition (int position, bool update)
 			{
 				Rectangle then = Bounds ();
 				this.position = position;
@@ -486,7 +516,9 @@ namespace FSpot {
 					selector.GdkWindow.InvalidateRect (then, false);
 					//selector.GdkWindow.InvalidateRect (now, false);
 				}
-				PositionChanged ();
+
+				if (update)
+					PositionChanged ();
 			}
 
 			private int position;
@@ -514,7 +546,7 @@ namespace FSpot {
 
 			public virtual bool IsInside (double x, double y)
 			{
-				return BoxTest (Bounds (), x, y);
+				return GroupSelector.IsInside (Bounds (), x, y);
 			}
 
 			public Manipulator (GroupSelector selector) 
@@ -632,7 +664,7 @@ namespace FSpot {
 					}
 				
 					Style.PaintHandle (selector.Style, selector.GdkWindow, State, ShadowType.In, 
-							    area, selector, "glass", bounds.X, inner.Y + inner. Height + border, 
+							    area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
 							    bounds.Width, handle_height, Orientation.Horizontal);
 
 					Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.Out, 
@@ -741,8 +773,7 @@ namespace FSpot {
 			Rectangle area; 
 			//Console.WriteLine ("expose {0}", args.Area);
 			foreach (Rectangle sub in args.Region.GetRectangles ()) {
-				area = sub;
-				if (args.Area.Intersect (background, out area)) {
+				if (sub.Intersect (background, out area)) {
 					Rectangle active = background;
 					int min_x = BoxX (min_limit.Position);
 					int max_x = BoxX (max_limit.Position + 1);
@@ -765,7 +796,7 @@ namespace FSpot {
 						   this, null, background.X, background.Y, 
 						   background.Width, background.Height);
 				
-				if (args.Area.Intersect (legend, out area)) {
+				if (sub.Intersect (legend, out area)) {
 					int i = 0;
 					
 					while (i < box_counts.Length)
@@ -774,16 +805,16 @@ namespace FSpot {
 				
 				if (has_limits) {
 					if (min_limit != null) {
-						min_limit.Draw (args.Area);
+						min_limit.Draw (sub);
 					}
 					
 					if (max_limit != null) {
-						max_limit.Draw (args.Area);
+						max_limit.Draw (sub);
 					}
 				}
 				
 				if (glass != null) {
-					glass.Draw (args.Area);
+					glass.Draw (sub);
 				}
 			}
 			return base.OnExposeEvent (args);
@@ -792,7 +823,7 @@ namespace FSpot {
 		protected override void OnSizeRequested (ref Requisition requisition)
 		{
 			requisition.Width = 500;
-			requisition.Height = 100;
+			requisition.Height = 70;
 			base.OnSizeRequested (ref requisition);
 		}
 
@@ -810,26 +841,74 @@ namespace FSpot {
 			
 			return (int) (max_height * 1.5);
 		}
+
+		private bool HandleScrollRight ()
+		{
+			Offset -= 10;
+			return true;
+		}
+
+		private bool HandleScrollLeft ()
+		{
+			Offset += 10;
+			return true;
+		}
 		
+		private void HandleLeftPressed (object sender, System.EventArgs ars)
+		{
+			HandleScrollLeft ();
+			left_delay.Start ();
+		}
+
+		private void HandleRightPressed (object sender, System.EventArgs ars)
+		{
+			HandleScrollRight ();
+			right_delay.Start ();
+		}
+
+		[GLib.ConnectBefore]
+		private void HandleScrollReleaseEvent (object sender, ButtonReleaseEventArgs args)
+		{
+			right_delay.Stop ();
+			left_delay.Stop ();
+		}
+
 		protected override void OnSizeAllocated (Gdk.Rectangle alloc)
 		{
 			base.OnSizeAllocated (alloc);
 			int legend_height = LengendHeight ();
+	
+			Gdk.Rectangle bar = new Rectangle (alloc.X + border, alloc.Y + border,
+							   alloc.Width - 2 *  border,
+							   alloc.Height - 2 * border - legend_height);
+							   
+		
+			if (left.Allocation.Y != bar.Y || left.Allocation.X != bar.X) {
+				left.SetSizeRequest (-1, bar.Height);
+				this.Move (left, bar.X - Allocation.X, bar.Y - Allocation.Y);
+			}
 			
-			background = new Rectangle (alloc.X + border, alloc.Y + border, 
-						    alloc.Width - 2 * border,
-						    alloc.Height - 2 * border - legend_height);
+			if (right.Allocation.Y != bar.Y || right.Allocation.X != bar.X + bar.Width - right.Allocation.Width) {
+				right.SetSizeRequest (-1, bar.Height);
+				this.Move (right,  bar.X - Allocation.X + bar.Width - right.Allocation.Width, 
+					   bar.Y - Allocation.Y);
+			}
+
+			background = new Rectangle (bar.X + left.Allocation.Width, bar.Y, 
+						    bar.Width - left.Allocation.Width - right.Allocation.Width, 
+						    bar.Height);
 			
 			legend = new Rectangle (background.X, background.Y + background.Height,
 						background.Width, legend_height);
 
-			if (event_window != null)
-				event_window.MoveResize (alloc.X, alloc.Y, alloc.Width, alloc.Height);
+			action = background.Union (legend);
 
-#if USE_BUTTONS
-			if (right.Allocation.X != 10 || (right.Allocation.Y - alloc.Y) != 10)
-				this.Move (right, 10, 10);
-#endif
+			if (event_window != null)
+				event_window.MoveResize (action.X, action.Y, action.Width, action.Height);
+			
+			this.Offset = this.Offset;
+
+			UpdateButtons ();
 		}
 
 		public GroupSelector () : base () 
@@ -840,17 +919,28 @@ namespace FSpot {
 			glass = new Glass (this);
 			min_limit = new Limit (this, Limit.LimitType.Min);
 			max_limit = new Limit (this, Limit.LimitType.Max);
-			min_limit.SetPosition (0);
-			max_limit.SetPosition (11);
 
-#if USE_BUTTONS
-			left = new Gtk.Button (Gtk.Stock.GoBack);
-			right = new Gtk.Button (Gtk.Stock.GoForward);
+			left = new Gtk.Button (new Gtk.Image (Gtk.Stock.GoBack, Gtk.IconSize.Button));
+			left.Relief = Gtk.ReliefStyle.None;
+			//left.Clicked += HandleScrollLeft;
+			left.Pressed += HandleLeftPressed;
+			left.ButtonReleaseEvent += HandleScrollReleaseEvent;
+			left_delay = new Delay (50, new GLib.IdleHandler (HandleScrollLeft));
+
+			right = new Gtk.Button (new Gtk.Image (Gtk.Stock.GoForward, Gtk.IconSize.Button));
+			right.Relief = Gtk.ReliefStyle.None;
+			right.Pressed += HandleRightPressed;
+			right.ButtonReleaseEvent += HandleScrollReleaseEvent;
+			right_delay = new Delay (50, new GLib.IdleHandler (HandleScrollRight));
+			//right.Clicked += HandleScrollRight;
+
 			this.Put (left, 0, 0);
 			this.Put (right, 100, 0);
 			left.Show ();
 			right.Show ();
-#endif
+			
+			Mode = RangeType.Min;
+			UpdateButtons ();
 		}
 
 		public GroupSelector (IntPtr raw) : base (raw) {}
