@@ -22,16 +22,15 @@ public class PhotoPopup {
 		
 		Gtk.Menu popup_menu = new Gtk.Menu ();
 		bool have_selection = count > 0;
-		bool have_multi = count > 1;
 
 		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Copy Image Location"), 
 				      new EventHandler (MainWindow.Toplevel.HandleCopyLocation), have_selection);
 		
 		GtkUtil.MakeMenuSeparator (popup_menu);
 
-		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Rotate Left"), "f-spot-rotate-270",
+		GtkUtil.MakeMenuItem (popup_menu, "f-spot-rotate-270",
 				      new EventHandler (MainWindow.Toplevel.HandleRotate270Command), have_selection);
-		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Rotate Right"), "f-spot-rotate-90",
+		GtkUtil.MakeMenuItem (popup_menu, "f-spot-rotate-90", 
 				      new EventHandler (MainWindow.Toplevel.HandleRotate90Command), have_selection);
 
 		GtkUtil.MakeMenuSeparator (popup_menu);
