@@ -25,52 +25,27 @@ class PixbufUtils {
 		return Math.Max (pixbuf.Width, pixbuf.Height);
 	}
 
-	public static int ComputeScaledWidth (Pixbuf pixbuf, int size)
-	{
-		int orig_width = (int) pixbuf.Width;
-		int orig_height = (int) pixbuf.Height;
-
-		if (orig_width > orig_height)
-			return size;
-		else
-			return (int) Math.Round ((int) size * ((double) orig_width / (double) orig_height));
-	}
-
-	public static int ComputeScaledHeight (Pixbuf pixbuf, int size)
-	{
-		int orig_width = (int) pixbuf.Width;
-		int orig_height = (int) pixbuf.Height;
-
-		if (orig_width > orig_height)
-			return (int) Math.Round ((int) size * ((double) orig_height / (double) orig_width));
-		else
-			return size;
-	}
-
-	public static void Fit (Pixbuf pixbuf,
-				int dest_width, int dest_height,
-				bool upscale_smaller,
-				out int fit_width, out int fit_height)
+	public static double Fit (Pixbuf pixbuf,
+				  int dest_width, int dest_height,
+				  bool upscale_smaller,
+				  out int fit_width, out int fit_height)
 	{
 		if (pixbuf.Width == 0 || pixbuf.Height == 0) {
 			fit_width = 0;
 			fit_height = 0;
-			return;
+			return 0.0;
 		}
 
-		if (pixbuf.Width <= dest_width && pixbuf.Height <= dest_height && ! upscale_smaller) {
-			fit_width = pixbuf.Width;
-			fit_height = pixbuf.Height;
-			return;
-		}
+		double scale = Math.Min (dest_width / (double)pixbuf.Width,
+					 dest_height / (double)pixbuf.Height);
+		
+		if (scale > 1.0 && !upscale_smaller)
+			scale = 1.0;
 
-		fit_width = dest_width;
-		fit_height = (int) Math.Round ((double) (pixbuf.Height * fit_width) / pixbuf.Width);
-
-		if (fit_height > dest_height) {
-			fit_height = dest_height;
-			fit_width = (int) Math.Round ((double) (pixbuf.Width * fit_height) / pixbuf.Height);
-		}
+		fit_width = (int)(scale * pixbuf.Width);
+		fit_height = (int)(scale * pixbuf.Height);
+		
+		return scale;
 	}
 
 
@@ -82,9 +57,6 @@ class PixbufUtils {
 		int max_height;
 		PixbufOrientation orientation;
 
-		// FIXME: this should be a property
-		public bool ScaleAlongLongestEdge = false;
-
 		public AspectLoader (int max_width, int max_height) 
 		{
 			this.max_height = max_height;
@@ -94,7 +66,6 @@ class PixbufUtils {
 
 		private void HandleSizePrepared (object obj, SizePreparedArgs args)
 		{
-			double scale;
 			switch (orientation) {
 			case PixbufOrientation.LeftTop:
 			case PixbufOrientation.LeftBottom:
@@ -108,16 +79,8 @@ class PixbufUtils {
 				break;
 			}
 
-			if (ScaleAlongLongestEdge) {
-				if (args.Width > args.Height)
-					scale = max_width / (double)args.Width;
-				else
-					scale = max_height / (double)args.Height;
-			} else {
-				scale = Math.Min (max_width / (double)args.Width,
-						  max_height / (double)args.Height);
-			}
-				
+			double scale = Math.Min (max_width / (double)args.Width,
+						 max_height / (double)args.Height);
 			
 			int scale_width = (int)(scale * args.Width);
 			int scale_height = (int)(scale * args.Height);
@@ -129,49 +92,50 @@ class PixbufUtils {
 		{
 			FileStream fs;
 
-			orientation = GetOrientation (path);
 			try {
+				orientation = GetOrientation (path);
 				fs = File.OpenRead (path);
+				int count;
+				
+				byte [] data = new byte [8192];
+				while (((count = fs.Read (data, 0, 8192)) > 0) && this.Write (data, (uint)count))
+					;
+				
+				this.Close ();
+				Gdk.Pixbuf rotated = TransformOrientation (this.Pixbuf, orientation, true);
+				
+				if (this.Pixbuf != rotated) {
+					CopyThumbnailOptions (this.Pixbuf, rotated);
+					this.Pixbuf.Dispose ();
+				}
+				return rotated;
 			} catch (Exception e) {
+				System.Console.Write ("Error loading image {0}", path);
 				return null;
 			}
 
-			int count;
-			
-			byte [] data = new byte [8192];
-			while (((count = fs.Read (data, 0, 8192)) > 0) && this.Write (data, (uint)count))
-				;
-			
-			this.Close ();
-			Gdk.Pixbuf rotated = TransformOrientation (this.Pixbuf, orientation, true);
-			
-			return rotated;
 		}
 	}
 
 	public static Pixbuf ScaleToMaxSize (Pixbuf pixbuf, int width, int height)
 	{
 		double scale = Math.Min  (width / (double)pixbuf.Width, height / (double)pixbuf.Height);
-	
-		
 		int scale_width = (int)(scale * pixbuf.Width);
 		int scale_height = (int)(scale * pixbuf.Height);
+
+		Gdk.Pixbuf result = pixbuf.ScaleSimple (scale_width, scale_height, Gdk.InterpType.Bilinear);
+
+		CopyThumbnailOptions (pixbuf, result);
+
 		return pixbuf.ScaleSimple (scale_width, scale_height, Gdk.InterpType.Bilinear);
 	}
-
+		
 	static public Pixbuf LoadAtMaxSize (string path, int max_width, int max_height)
 	{
 		PixbufUtils.AspectLoader loader = new AspectLoader (max_width, max_height);
 		return loader.LoadFromFile (path);
 	}
 
-	static public Pixbuf LoadAtMaxEdgeSize (string path, int longest_edge)
-	{
-		PixbufUtils.AspectLoader loader = new AspectLoader (longest_edge, longest_edge);
-		loader.ScaleAlongLongestEdge = true;
-		return loader.LoadFromFile (path);
-	}
-
 	static private Pixbuf LoadFromStream (System.IO.Stream input)
 	{
 		Gdk.PixbufLoader loader = new Gdk.PixbufLoader ();
@@ -195,8 +159,44 @@ class PixbufUtils {
 	
 	public static bool SetOption(Gdk.Pixbuf pixbuf, string key, string value)
 	{
-		bool ret = gdk_pixbuf_set_option(pixbuf.Handle, key, value);
-		return ret;
+		
+		if (value != null)
+			return gdk_pixbuf_set_option(pixbuf.Handle, key, value);
+		else
+			return false;
+	}
+	
+	public static void CopyThumbnailOptions (Gdk.Pixbuf src, Gdk.Pixbuf dest)
+	{
+		if (src != null && dest != null) {
+			PixbufUtils.SetOption (dest, "tEXt::Thumb::URI", src.GetOption ("tEXt::Thumb::URI"));
+			PixbufUtils.SetOption (dest, "tEXt::Thumb::MTime", src.GetOption ("tEXt::Thumb::MTime"));
+		}
+	}
+
+	[DllImport("libgdk_pixbuf-2.0-0.dll")]
+	static extern bool gdk_pixbuf_save_to_bufferv (IntPtr raw, out IntPtr data, out uint length, string type, 
+						       string [] keys, string [] values, out IntPtr error);
+					
+	public static byte [] Save (Gdk.Pixbuf pixbuf, string type, string [] options, string [] values)
+	{
+		IntPtr error = IntPtr.Zero;
+		IntPtr data;
+		uint length;
+		gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
+					    out data, 
+					    out length, 
+					    type,
+					    options, values,
+					    out error);
+
+		if (error != IntPtr.Zero) 
+			throw new GLib.GException (error);
+
+		byte [] content = new byte [length];
+		Marshal.Copy (data, content, 0, (int)length);
+
+		return content;
 	}
 	
 	public static Pixbuf TagIconFromPixbuf (Pixbuf source)
@@ -223,15 +223,6 @@ class PixbufUtils {
 	}
 		
 
-	static public Pixbuf LoadFromAssembly (System.Reflection.Assembly assembly, string resource)
-	{
-		System.IO.Stream s = assembly.GetManifestResourceStream (resource);
-		if (s == null)
-			return null;
-		else
-			return LoadFromStream (s);
-	}
-
 	static public Pixbuf LoadFromScreen () {
 		Screen screen = Display.Default.GetScreen (0);
 		Drawable d = screen.RootWindow;
@@ -252,9 +243,98 @@ class PixbufUtils {
 
 	static public Pixbuf LoadFromAssembly (string resource)
 	{
-		return LoadFromAssembly (System.Reflection.Assembly.GetCallingAssembly (), resource);
+		try {
+			return new Pixbuf (System.Reflection.Assembly.GetCallingAssembly (), resource);
+		} catch {
+			return null;
+		}
 	}
 
+	[DllImport ("libc")]
+	static extern int rename (string oldpath, string newpath);
+
+	public static void SaveAtomic (Gdk.Pixbuf src, string filename, string type, string [] keys, string [] values)
+	{
+			string tmpname = filename + ".tmp";
+			src.Savev (tmpname, type, keys, values);
+			if (rename (tmpname, filename) < 0)
+				throw new Exception ("Error renaming file");
+	}
+
+	public static Gdk.Pixbuf ScaleToAspect (Gdk.Pixbuf orig, int width, int height)
+	{
+		Gdk.Rectangle pos;
+		double scale = Fit (orig, width, height, false, out pos.Width, out pos.Height);
+		pos.X = (width - pos.Width) / 2;
+		pos.Y = (height - pos.Height) / 2;
+
+		Pixbuf scaled = new Pixbuf (Colorspace.Rgb, false, 8, width, height);
+		scaled.Fill (0x000000); 
+
+		System.Console.WriteLine ("pos= {0}", pos.ToString ());
+
+		orig.Composite (scaled, pos.X, pos.Y, 
+				pos.Width, pos.Height,
+				pos.X, pos.Y, scale, scale,
+				Gdk.InterpType.Bilinear,
+				255);
+
+		return scaled;
+	}
+
+	[StructLayout(LayoutKind.Sequential)]
+	public unsafe struct FPixbufJpegMarker {
+		public int type;
+		public byte *data;
+		public int length;
+	}
+
+	[DllImport ("libfspot")]
+	static extern bool f_pixbuf_save_jpeg (IntPtr src, string path, int quality, FPixbufJpegMarker [] markers, int num_markers);
+
+	public static void SaveJpeg (Pixbuf pixbuf, string path, int quality, Exif.ExifData exif_data)
+	{
+		// The DCF spec says thumbnails should be 160x120 always
+		Pixbuf thumbnail = ScaleToAspect (pixbuf, 160, 120);
+		byte [] thumb_data = Save (thumbnail, "jpeg", null, null);
+		exif_data.Data = thumb_data;
+		thumbnail.Dispose ();
+
+		// Most of the things we will set will be in the 0th ifd
+		Exif.ExifContent content = exif_data.GetContents (Exif.ExifIfd.Zero);
+
+		// reset the orientation tag the default is top/left
+		content.GetEntry (Exif.ExifTag.Orientation).Reset ();
+
+		// set the write time in the datetime tag
+		content.GetEntry (Exif.ExifTag.DateTime).Reset ();
+
+		// set the software tag
+		content.GetEntry (Exif.ExifTag.Software).SetData (FSpot.Defines.PACKAGE + " version " + FSpot.Defines.VERSION);
+
+		byte [] data = exif_data.Save ();
+		FPixbufJpegMarker [] marker = new FPixbufJpegMarker [0];
+		bool result = false;
+
+		unsafe {
+			if (data.Length > 0) {
+				
+				fixed (byte *p = data) {
+					marker = new FPixbufJpegMarker [1];
+					marker [0].type = 0xe1; // APP1 marker
+					marker [0].data = p;
+					marker [0].length = data.Length;
+					
+					result = f_pixbuf_save_jpeg (pixbuf.Handle, path, quality, marker, marker.Length);
+				}					
+			} else
+				result = f_pixbuf_save_jpeg (pixbuf.Handle, path, quality, marker, marker.Length);
+			
+		}
+
+		if (result == false)
+			throw new System.Exception ("Error Saving File");
+	}
 
 	[DllImport ("libfspot")]
 	static extern IntPtr f_pixbuf_unsharp_mask (IntPtr src, double radius, double amount, double threshold);
@@ -306,6 +386,7 @@ class PixbufUtils {
 		bchsw.Dispose ();
 	}
 
+
 	public static unsafe void ColorAdjust (Gdk.Pixbuf src, Gdk.Pixbuf dest, Cms.Transform trans)
 	{
 		int width = src.Width;
@@ -320,24 +401,80 @@ class PixbufUtils {
 		
 	}
 
-	public static PixbufOrientation GetOrientation (string path)
+	public static Gdk.Pixbuf GetThumbnail (ExifData data)
 	{
-		ExifData exif = new ExifData (path);
-		byte [] value = exif.LookupData (ExifTag.Orientation);
-		PixbufOrientation orientation = PixbufOrientation.TopLeft;
+		byte [] thumb_data = data.Data;
+
+		if (thumb_data.Length > 0) {
+			PixbufOrientation orientation = GetOrientation (data);
+			MemoryStream mem = new MemoryStream (thumb_data);
+			Gdk.Pixbuf thumb = new Gdk.Pixbuf (mem);
+			Gdk.Pixbuf rotated = PixbufUtils.TransformOrientation (thumb, orientation);
+			
+			if (rotated != thumb)
+				thumb.Dispose ();
+			
+			return rotated;
+		}
+		return null;
+	}
+
+	public static PixbufOrientation GetOrientation (ExifData data)
+        {
+               byte [] value = data.LookupData (ExifTag.Orientation);
+                PixbufOrientation orientation = PixbufOrientation.TopLeft;
 
 		if (value != null) {
-			System.Console.WriteLine ("len = {0} val [0] = {1} string = {2}", value.Length, 
-						  value[0], exif.LookupString (ExifTag.Orientation));
 			orientation = (PixbufOrientation)value [0];
 		}
+		
+		return orientation;
+	}
+
+	public static PixbufOrientation GetOrientation (Exif.ExifData data)
+	{
+		PixbufOrientation orientation = PixbufOrientation.TopLeft;
+		
+		Exif.ExifEntry e = data.GetContents (Exif.ExifIfd.Zero).Lookup (Exif.ExifTag.Orientation);
+
+		if (e != null) {
+			ushort [] value = e.GetDataUShort ();
+			orientation = (PixbufOrientation) value [0];
+		}
 
 		return orientation;
 	}
+	
+	public static PixbufOrientation GetOrientation (string path)
+	{
+		Exif.ExifData data = new Exif.ExifData (path);
+
+		PixbufOrientation orientation = GetOrientation (data);
+		data.Dispose ();
+
+		return orientation;
+	}
+
+	[DllImport("gnomeui-2")]
+	static extern IntPtr gnome_thumbnail_scale_down_pixbuf(IntPtr pixbuf, int dest_width, int dest_height);
+
+	public static Gdk.Pixbuf ScaleDown (Gdk.Pixbuf src, int width, int height)
+	{
+		IntPtr raw_ret = gnome_thumbnail_scale_down_pixbuf(src.Handle, width, height);
+		Gdk.Pixbuf ret;
+		if (raw_ret == IntPtr.Zero)
+			ret = null;
+		else
+			ret = (Gdk.Pixbuf) GLib.Object.GetObject(raw_ret, true);
+		return ret;
+	}
 
 	public static Gdk.Pixbuf TransformOrientation (Gdk.Pixbuf src, PixbufOrientation orientation, bool copy_data)
 	{
 		Gdk.Pixbuf pixbuf;
+		if (src == null)
+			return null;
+		
 		switch (orientation) {
 		case PixbufOrientation.LeftTop:
 		case PixbufOrientation.LeftBottom:
@@ -370,81 +507,97 @@ class PixbufUtils {
 		return TransformOrientation (src, orientation, true);
 	}
 
-	public static Gdk.Rectangle TransformAndCopy (Gdk.Pixbuf src, Gdk.Pixbuf dest, PixbufOrientation orientation, Gdk.Rectangle args)
+	public static Gdk.Rectangle TransformOrientation (Gdk.Pixbuf src, Gdk.Rectangle args, PixbufOrientation orientation)
+	{
+		return TransformOrientation (src.Width, src.Height, args, orientation);
+	}
+	
+	public static Gdk.Rectangle TransformOrientation (int total_width, int total_height, Gdk.Rectangle args, PixbufOrientation orientation)
 	{
 		Gdk.Rectangle area = args;
-		Gdk.Pixbuf region;
-		Gdk.Pixbuf altered;
 		
 		switch (orientation) {
-		case PixbufOrientation.LeftBottom:
-			area.X = args.Y;
-			area.Y = src.Width - args.X - args.Width;
+		case PixbufOrientation.BottomRight:
+			area.X = total_width - args.X - args.Width;
+			area.Y = total_height - args.Y - args.Height;
+			break;
+		case PixbufOrientation.TopRight:
+			area.X = total_width - args.X - args.Width;
+			break;
+		case PixbufOrientation.BottomLeft:
+			area.Y = total_height - args.Y - args.Height;
+			break;
+		case PixbufOrientation.LeftTop:
+			area.X = area.Y;
+			area.Y = area.X;
+			area.Width = args.Height;
+			area.Height = args.Width;
+			break;
+		case PixbufOrientation.RightBottom:
+			area.X = total_height - args.Y - args.Height;
 			area.Width = args.Height;
 			area.Height = args.Width;
-			
-			region = new Gdk.Pixbuf (src, args.X, args.Y,
-						 args.Width, args.Height);
-			altered = PixbufUtils.Rotate90 (region, true);
-
-			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
-
-			region.Dispose ();
-			altered.Dispose ();
 			break;
 		case PixbufOrientation.RightTop:
-			area.X = src.Height - args.Y - args.Height;
+			area.X = total_height - args.Y - args.Height;
 			area.Y = args.X;
 			area.Width = args.Height;
 			area.Height = args.Width;
-			
-			region = new Gdk.Pixbuf (src, args.X, args.Y,
-						 args.Width, args.Height);
-			altered = PixbufUtils.Rotate90 (region, false);
-
-			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
-
-			region.Dispose ();
-			altered.Dispose ();			
 			break;
-		case PixbufOrientation.BottomRight:
-			area.X = src.Width - args.X - args.Width;
-			area.Y = src.Height - args.Y - args.Height;
-
-			region = new Gdk.Pixbuf (src, args.X, args.Y,
-						 args.Width, args.Height);
-			altered = PixbufUtils.Mirror (region, true, true);
-			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
-
-			region.Dispose ();
-			altered.Dispose ();
-			break;
-		case PixbufOrientation.TopRight:
-			area.X = src.Width - args.X - args.Width;
-
-			region = new Gdk.Pixbuf (src, args.X, args.Y,
-						 args.Width, args.Height);
-			altered = PixbufUtils.Mirror (region, true, false);
-			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
-
-			region.Dispose ();
-			altered.Dispose ();
-			break;
-		case PixbufOrientation.BottomLeft:
-			area.Y = src.Height - args.Y - args.Height;
-
-			region = new Gdk.Pixbuf (src, args.X, args.Y,
-						 args.Width, args.Height);
-			altered = PixbufUtils.Mirror (region, false, true);
-			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
-
-			region.Dispose ();
-			altered.Dispose ();
+		case PixbufOrientation.LeftBottom:
+			area.X = args.Y;
+			area.Y = total_width - args.X - args.Width;
+			area.Width = args.Height;
+			area.Height = args.Width;
 			break;
 		default:
-			src.CopyArea (area.X, area.Y, area.Width, area.Height, dest, area.X, area.Y);
-			return area;
+			break;
 		}
+		
+		return area;
+	}
+	
+	public static Gdk.Rectangle TransformAndCopy (Gdk.Pixbuf src, Gdk.Pixbuf dest, PixbufOrientation orientation, Gdk.Rectangle args)
+	{
+		Gdk.Rectangle area = TransformOrientation (src, args, orientation);
+
+		int step = 512;
+
+		Gdk.Rectangle rect = new Gdk.Rectangle (args.X, args.Y, 
+							Math.Min (step, args.Width),
+							Math.Min (step, args.Height));
+
+		Gdk.Pixbuf tmp = new Gdk.Pixbuf (src.Colorspace, src.HasAlpha, 
+						 src.BitsPerSample,
+						 rect.Height, rect.Width);
+
+		Gdk.Rectangle subarea;
+		while (rect.Y < args.Y + args.Height) {
+			while (rect.X < args.X + args.Width) {
+				rect.Intersect (args, out subarea);
+				Gdk.Rectangle trans = TransformOrientation (src, subarea, orientation);
+
+			        Gdk.Pixbuf ssub = new Gdk.Pixbuf (src, subarea.X, subarea.Y,
+								  subarea.Width, subarea.Height);
+
+				
+				Gdk.Pixbuf tsub = new Gdk.Pixbuf (tmp, 0, 0, trans.Width, trans.Height);
+
+				CopyWithOrientation (ssub, tsub, orientation);
+				
+				tsub.CopyArea (0, 0, trans.Width, trans.Height, dest, trans.X, trans.Y);
+				
+
+				
+				ssub.Dispose ();
+				tsub.Dispose ();
+				rect.X += rect.Width;
+			}
+			rect.X = args.X;
+			rect.Y += rect.Height;
+		}
+
+		tmp.Dispose ();
 		return area;
 	}
 
@@ -486,4 +639,21 @@ class PixbufUtils {
 			throw new GLib.GException (error);
 		}
 	}
+
+	[DllImport ("libfspot")]
+	static extern void f_pixbuf_copy_with_orientation (IntPtr src, IntPtr dest, int orientation);
+
+	public static void CopyWithOrientation (Gdk.Pixbuf src, Gdk.Pixbuf dest, PixbufOrientation orientation)
+	{
+		f_pixbuf_copy_with_orientation (src.Handle, dest.Handle, (int)orientation);
+	}
+
+#if false
+	[DllImport("glibsharpglue")]
+	static extern int gtksharp_object_get_ref_count (IntPtr obj);
+	
+	public static int RefCount (GLib.Object obj) {
+		return gtksharp_object_get_ref_count (obj.Handle);
+	}
+#endif
 }
